#include "can.h"

void CAN_Config(void){
    /* Enable the CAN module */
    C1CONHbits.ON = 1;
    
    //config mode
    C1CONTbits.REQOP = 4;
    while(C1CONUbits.OPMOD != 4);
    
    /* Initialize the C1FIFOBA with the start address of the CAN FIFO message object area. */
    C1FIFOBA = 0x3800;
    // CLKSEL0 disabled; PXEDIS enabled; ISOCRCEN enabled; DNCNT 0; 
    C1CONL = 0x60;
    // ON enabled; FRZ disabled; SIDL disabled; BRSDIS disabled; WFT T11 Filter; WAKFIL enabled; 
    C1CONH = 0x87;
    // TXQEN enabled; STEF disabled; SERR2LOM disabled; ESIGM disabled; RTXAT disabled; 
    C1CONU = 0x10;

    // SJW 15; 
    C1NBTCFGL = 0x0F;
    // TSEG2 15; 
    C1NBTCFGH = 0x0F;
    // TSEG1 46; 
    C1NBTCFGU = 0x2E;
    // BRP 0; 
    C1NBTCFGT = 0x00;
    // SJW 5; 
    C1DBTCFGL = 0x05;
    // TSEG2 5; 
    C1DBTCFGH = 0x05;
    // TSEG1 8; 
    C1DBTCFGU = 0x08;
    // BRP 0; 

    C1DBTCFGT = 0x00;
    // TDCO 9; 
    C1TDCH = 0x09;
    // TDCMOD Auto; 
    C1TDCU = 0x02;
    // TXATIE disabled; TXQEIE disabled; TXQNIE disabled; 
    C1TXQCONL = 0x00;
    // FRESET enabled; UINC disabled; 
    C1TXQCONH = 0x04;
    // TXAT 3; TXPRI 1; 
    C1TXQCONU = 0x60;
    // PLSIZE 64; FSIZE 12; 
    C1TXQCONT = 0xEB;
    // TXEN disabled; RTREN disabled; RXTSEN disabled; TXATIE disabled; RXOVIE disabled; TFERFFIE disabled; TFHRFHIE disabled; TFNRFNIE disabled; 
    C1FIFOCON1L = 0x00;
    // FRESET enabled; TXREQ disabled; UINC disabled; 
    C1FIFOCON1H = 0x04;
    // TXAT Unlimited number of retransmission attempts; TXPRI 1; 
    C1FIFOCON1U = 0x60;
    // PLSIZE 64; FSIZE 16; 
    C1FIFOCON1T = 0xEF;

    C1FLTCON0L = 0x81;
    

    C1CONTbits.REQOP = 0;
    while (C1CONUbits.OPMOD != 0);
}

uint8_t CAN1_Receive(CAN_MSG *CanMsg){

    if ((C1FIFOSTA1L & 0x1)){        //fifo is not empty
        CAN_MSG *rxFifoObj = (CAN_MSG *) C1FIFOUA1;
        CanMsg->status = rxFifoObj->status;
        
        if(CanMsg->IDE == 1)   //Extended
            CanMsg->ID = (rxFifoObj->ID >> 11) | ((rxFifoObj->ID & 0x7FF) << 18);
        else                  //Standard
            CanMsg->ID = rxFifoObj->ID & 0x7FF;
        
        for(uint8_t i = 0; i < DLC_BYTES[CanMsg->DLC]; i++)
            CanMsg->Byte[i] = rxFifoObj->Byte[i];
            
        C1FIFOCON1H |= _C1FIFOCON1H_UINC_MASK;
        return 1;
    }
    return 0;
}

uint8_t CAN1_Transmit(CAN_MSG *CanMsg){
    CAN_MSG *txFifoObj = (CAN_MSG *) C1TXQUA;
     
    if (C1TXQSTAL & 0x1){
        txFifoObj->status = CanMsg->status;
        
        if(CanMsg->IDE == 1)   //Extended
            txFifoObj->ID = CanMsg->ID<<11 | ((CanMsg->ID>>18) & 0x3FFFF);
        else                   //Standard
            txFifoObj->ID = CanMsg->ID;
        
        for(uint8_t i = 0; i < DLC_BYTES[CanMsg->DLC]; i++)
            txFifoObj->Byte[i] = CanMsg->Byte[i];
        
        C1TXQCONH |= (_C1FIFOCON1H_TXREQ_MASK | _C1FIFOCON1H_UINC_MASK);
        return 1;
    }
    return 0;
}
