
#include <xc.h>
#include "config.h"
#include "can.h"
#include "memory.h"

#define _XTAL_FREQ 64000000

//extern uint16_t bufferRAM;
//uint16_t *p_bufferRAM = &bufferRAM;

uint8_t *BufferRAM = (uint16_t*) & bufferRAM;

void Boot_Send_ID();
void OSCILLATOR_Initialize(void);


uint32_t Device_ID;
CAN_MSG botschaft1;


static void (*APP)(void) = 0x8000;

void main(void) {    
    if(FLASH_ReadWord(0x8000) != 0xFFFF)
        APP();
        
    Device_ID = FLASH_ReadWord(0x200000);
        
    OSCILLATOR_Initialize();
    LATA = 0x00;
    LATB = 0x20;
    LATC = 0x00;

    TRISE = 0x07;
    TRISA = 0xFF;
    TRISB = 0xDB;
    TRISC = 0xEF;

    ANSELC = 0xEF;
    ANSELB = 0xD7;
    ANSELA = 0xFF;
    
    RB2PPS = 0x46;   //RB2->CAN1:CANTX;    
    CANRXPPS = 0x0B;   //RB3->CAN1:CANRX;
    
    CAN_Config();
    
    Boot_Send_ID();
    
    LATBbits.LATB5 = 0;
    
    while(1){
        __delay_ms(100);
        LATBbits.LATB5 ^= 1;
        
        if(CAN1_Receive(&botschaft1) == 1){
            if(botschaft1.ID >> 16 == 0x1FFF){   //Bootmen botschaft
                uint8_t Command = botschaft1.ID & 0xF;
                uint32_t CanId = (botschaft1.ID >> 4) & 0xFFF;
                
                if(CanId == Device_ID | CanId == 0xFFF){    //Eigene ID oder Brodcast
            
                    if(Command == 0){    //Info
                        Boot_Send_ID();
                    }

                    if(Command == 1){    //Change to App   
                        Reset();
                        while(1);
                    }
                    
                   //Command == 2 Change to Boot
                    
                    if(Command == 3){       //Write Word
                        FLASH_WriteWord(botschaft1.DWord[0], botschaft1.Word[2]);
                        if (NVMCON1bits.WRERR){
                            LATBbits.LATB5 ^= 1;
                        }
                    }
                    
                    if(Command == 4){       //Erase Page
                        FLASH_EraseBlock(botschaft1.DWord[0]);
                    }
                    
                    if(Command == 5){
                        
                        FLASH_EraseBlock(botschaft1.DWord[0]);
                        
                        for(uint16_t i = 0; i<0x100; i++)
                            BufferRAM[i] = i;
                        
                        FLASH_WritePage(botschaft1.DWord[0]);
                        
                    }
                    
                    if(Command == 0xF){     //Read Page
                        FLASH_ReadPage(0x200000);
                        
                        botschaft1.ID = 0x1FFF0002 + (Device_ID << 4);     //Boot
                        botschaft1.IDE = 0;
                        botschaft1.FDF = 1;
                        botschaft1.BRS = 1;
                        botschaft1.DLC = 0xF;

                        for(uint16_t i = 0; i<0x100; i++){
                            botschaft1.Byte[i & 0x3F] = BufferRAM[i];
                            
                            if((i & 0x3F) == 0x3F){
                                botschaft1.ID = i & 0xC0;
                                CAN1_Transmit(&botschaft1);
                            }
                        }
                    }
                }
            }
        }
    }
    
    return;
}

void Boot_Send_ID(){
    botschaft1.ID = 0x1FFF0002 + (Device_ID << 4);     //Boot
    botschaft1.IDE = 1;
    botschaft1.FDF = 1;
    botschaft1.BRS = 1;
    botschaft1.DLC = 0xF;

    FLASH_ReadPage(0x200000);
    
    for(uint8_t i = 0; i<63; i++){
        botschaft1.Byte[i] = BufferRAM[i];
    }
    botschaft1.Byte[63] = 0;

    CAN1_Transmit(&botschaft1);
}

void OSCILLATOR_Initialize(void)
{
    // NOSC EXTOSC   with 4x PLL; NDIV 1; 
    OSCCON1 = 0x20;
    // CSWHOLD may proceed; SOSCPWR Low power; 
    OSCCON3 = 0x00;
    // MFOEN disabled; LFOEN disabled; ADOEN disabled; PLLEN enabled; SOSCEN disabled; EXTOEN disabled; HFOEN disabled; 
    OSCEN = 0x01;
    // HFFRQ 4_MHz; 
    OSCFRQ = 0x02;
    // TUN 0; 
    OSCTUNE = 0x00;
    // Wait for PLL to stabilize
    while(PLLR == 0){
    }
}
